//+------------------------------------------------------------------+
//|                                           Damiani_volatmeter.mq4 |
//|                         Copyright  2006, Luis Guilherme Damiani |
//|                                      http://www.damianifx.com.br |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Luis Guilherme Damiani"
#property link      "http://www.damianifx.com.br"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  Silver
#property indicator_color2  FireBrick
#property indicator_color3  Lime
#property indicator_width1  2
#property indicator_width2  1
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern int    Viscosity       = 7;
extern int    Sedimentation   = 50;
extern double Threshold_level = 1.1;
extern bool   lag_supressor   = true;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double thresholdBuffer[];
double vol_m[];
double vol_t[];
double ind_c[];
double trend[];

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{

   IndicatorBuffers(5);
   SetIndexBuffer(0,thresholdBuffer); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,vol_m);           SetIndexStyle(1,DRAW_SECTION);
   SetIndexBuffer(2,vol_t);           SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(3,ind_c);         
   SetIndexBuffer(4,trend);
   
      //
      //
      //
      //
      //
      
      indicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
      returnBars        = (TimeFrame=="returnBars");    if (returnBars) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
return(0);
}  

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//

int start()
{
    int counted_bars = IndicatorCounted();
    int limit,i;

    if(counted_bars<0) return(-1);
    if(counted_bars>0) counted_bars--;
          limit = MathMin(Bars-counted_bars,Bars-1);
          if (limit > Sedimentation+5)limit = limit-Sedimentation;
          if (returnBars)  { thresholdBuffer[0] = limit+1; return(0); }  
    //
    //
    //
    //
    //
    
    if (calculateValue || timeFrame == Period())
    {
      for(i=limit; i>=0; i--)
      {
        double sa  = iATR(NULL,0,Viscosity,i);
        double atr = NormalizeDouble(sa,Digits);
        
        if(lag_supressor)
          ind_c[i] = sa/iATR(NULL,0,Sedimentation,i)+0.5 * (ind_c[i+1] - ind_c[i+3]);   
        else
          ind_c[i] = sa/iATR(NULL,0,Sedimentation,i); 
        
        thresholdBuffer[i] = Threshold_level - iStdDev(NULL,0,Viscosity,    0,MODE_LWMA,PRICE_TYPICAL,i)/
                                               iStdDev(NULL,0,Sedimentation,0,MODE_LWMA,PRICE_TYPICAL,i); 
        
        
        trend[i] = trend[i+1];
        if (ind_c[i] > thresholdBuffer[i]) trend[i]= 1; 
        if (ind_c[i] < thresholdBuffer[i]) trend[i]=-1;
        if (trend[i] > 0)                  { vol_t[i] = ind_c[i]; vol_m[i] = ind_c[i];  
          IndicatorShortName(timeFrameToString(timeFrame)+" DAMIANI Signal/Noise: TRADE  /  ATR= "+DoubleToStr(atr,Digits)+"    values:");}
        else                               { vol_t[i] = ind_c[i]; vol_m[i] = EMPTY_VALUE;
          IndicatorShortName(timeFrameToString(timeFrame)+" DAMIANI Signal/Noise: DO NOT trade  /  ATR= "+DoubleToStr(atr,Digits)+"    values:");}   
         
      
      }
      manageAlerts();
   return (0);
   } 
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0; i--)
   {
       int y = iBarShift(NULL,timeFrame,Time[i]);
            thresholdBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Viscosity,Sedimentation,Threshold_level,lag_supressor,0,y);
            vol_m[i]           = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Viscosity,Sedimentation,Threshold_level,lag_supressor,1,y);
            vol_t[i]           = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Viscosity,Sedimentation,Threshold_level,lag_supressor,2,y);
            ind_c[i]           = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Viscosity,Sedimentation,Threshold_level,lag_supressor,3,y);
            trend[i]           = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Viscosity,Sedimentation,Threshold_level,lag_supressor,4,y);
            if (ind_c[i] > thresholdBuffer[i]) trend[i]= 1; 
            if (ind_c[i] < thresholdBuffer[i]) trend[i]=-1;
            if (trend[i] > 0)                  { vol_t[i] = ind_c[i]; vol_m[i] = ind_c[i];  
              IndicatorShortName(timeFrameToString(timeFrame)+" DAMIANI Signal/Noise: TRADE  /  ATR= "+DoubleToStr(atr,Digits)+"    values:");}
            else                               { vol_t[i] = ind_c[i]; vol_m[i] = EMPTY_VALUE;
              IndicatorShortName(timeFrameToString(timeFrame)+" DAMIANI Signal/Noise: DO NOT trade  /  ATR= "+DoubleToStr(atr,Digits)+"    values:");}           
    }
    manageAlerts();   
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"trend");
         if (trend[whichBar] ==-1) doAlert(whichBar,"no trend");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Damiani voltmeter ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Damiani voltmeter "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

